#==================================================================
#
#   STATISTICAL PRACTICE IN EPIDEMIOLOGY WITH R, Tartu 2007
# 
#   Practical 1.9:  Survival analysis: The Stroke dataset 
#   -------------   Monday 28 May, KF & EL
#     
#------------------------------------------------------------------

## 1.  Loading the dataset, assumed to be in the working directory 
## 
##     Note: field separator is ";", and
##           missing values denoted by "."

stroke <- read.csv2("../data/stroke.csv", header=T, na.strings=".")
str(stroke)
summary(stroke)

##-----------------------------------------------------------------
##
## 2. Creating time and status variables
##

##    First convert the dates.
##    Note the date format in the dataset: e.g "30.10.1993"

stroke$DSTR <- as.Date(stroke$DSTR, format="%d.%m.%Y")  
stroke$DIED <- as.Date(stroke$DIED, format="%d.%m.%Y")

## For those with missing date of death, follow-up ended in 01/01/1996 

stroke$stop <- as.Date("01.01.1996", format="%d.%m.%Y")

## Note that a few dates of death are after the stop date:
subset( stroke, DIED > stop )
## These persons must have reset their exit date and be treated as censored
stroke$DIED[stroke$DIED>stroke$stop] <- NA

## Pay attention to a conditional assignment in the following.
## Note also the indirect way to express that the value IS available!

stroke$event <- as.numeric( !is.na(stroke$DIED) )
stroke$time <- ifelse( stroke$event == 1, 
         stroke$DIED - stroke$DSTR, stroke$stop - stroke$DSTR)/365.25 
stroke[1:5,]
summary(stroke)

##---------------------------------------------------------------------
##
##  3. Kaplan-Meier curves
##
 
library(survival)
library(Epi)

##   Define first the survival object.
##   This is a useful thing to do at the outset, so that the 
##   lengthy 'Surv(time, event)' can later on be avoided  
stroke$st <- Surv(stroke$time, stroke$event)
str(stroke$st)
summary(stroke$st)

##  Overall survival: Save the results into an object, from
##  which you then can extract results of interest
s.all <- survfit(st ~ 1, data=stroke)

##  The median is reported by the print method:
s.all
str(s.all)     # contents of the object

##  Kaplan-Meier curve with pointwise confidence intevals
plot(s.all)
##  summary method gives the whole 'life table'
summary(s.all)

##  Finally: KM-curves and median survival times by diagnosis

s.dgn <- survfit(st ~ DGN, data=stroke)
plot(s.dgn, col=1:4, legend.text=levels(stroke$DGN))
s.dgn

##-------------------------------------------------------------
##
##  4. Log-cumulative hazards plots
##
plot(s.dgn, col=1:4, fun="cloglog", xlim=c(0.002,5.5))

##-------------------------------------------------------------
##
##  5. Comparing men and women
##
s.sex <- survfit(st~SEX, data=stroke)
plot(s.sex, col=1:2, legend.text=c("Female","Male"))
survdiff(st ~ SEX, data=stroke)

## Do females really have a higher risk of dying after stroke?

##---------------------------------------------------------------
##
##  6. Cox PH model with age, sex, and age & sex
##

m.age <- coxph(st ~ I((AGE-70)/10), data=stroke) # age only
summary( m.age )
round( ci.lin(m.age, Exp=T)[ , 5:7], 4 )

m.sex <- coxph(st ~ SEX, data=stroke)            # sex only
round( ci.lin(m.sex, Exp=T)[ , 5:7], 4 )

m.ax <- coxph(st ~ I((AGE-70)/10) + SEX, data=stroke) # both
summary(m.ax)
round(ci.lin(m.ax, Exp=T)[, 5:7], 4)

##  Categorize age into three age groups
stroke$agegr <- cut(stroke$AGE, br=c(0,65,80,100))
table(stroke$agegr)

##  Plot survival jointly by age group and sex 
s.agrx <- survfit(st ~ agegr + SEX, data=stroke)
##  Three age groups have different line types,
##  within these, men and women have different colours:
plot(s.agrx, col=rep(1:2,3), lty=c(1,1,2,2,3,3))

##  The reason why men and women have different survival:
##  Women were older at diagnosis than men
tapply(stroke$AGE, stroke$SEX, summary)

##--------------------------------------------------------------
##
##  7. Fitting a model with all "significant" covariates
##
m1 <- coxph(st ~ DGN + I((AGE-70)/10) + COMA + DIAB + MINF,
         data = stroke)
round( ci.lin(m1, Exp=T)[, 5:7], 4) 
cox.zph(m1)

##  Stratification by diagnostic group
m2 <- coxph(st ~ strata(DGN) + I((AGE-70)/10) + COMA + DIAB + MINF,
        data=stroke)
cox.zph(m2)  ## proportionality still problematic!!!

##---------------------------------------------------------------
##
##  8. Predicted survival curves for those aged 75 y
##    (ignoring proportionality assumption being not fulfilled)
## 
m3 <- coxph(st ~ strata(DGN) + I((AGE-70)/10), data=stroke)
plot(survfit(m3, newdata=list(AGE=75)), col=1:4)

##  For ages 70y and 80y:
plot(survfit(m3, newdata=list( AGE=c(70,80)) ), col=rep(1:4, rep(2,4)))

##------------------------------------------------------------------
##
##   9. Survival by hypertension:
##
m4 <- coxph(st ~ HAN, data=stroke)
plot(survfit(m4, newdata = list( HAN = c(0,1) ) ), col=1:2)

##------------------------------------------------------------------
##
## Epilogue: A PERHAPS BETTER WAY TO ANALYZE THIS DATASET
## 
## Think about the nature of the disease - the hazard can be 
## stroke-related only for a short time period after the stroke
##
## Let's analyze survival during the first month only
##
stroke$event1 <- stroke$event*(stroke$time < 1/12)
stroke$time1 <-  ifelse(stroke$time < 1/12, stroke$time, 1/12)

stroke$st1 <- Surv(stroke$time1, stroke$event1)
m5 <- coxph( st1 ~ DGN + I((AGE-70)/10) + DIAB + MINF,
   data=stroke)
cox.zph(m5)

ag <- cut(stroke$AGE, br=c(0,60,75,100))  
m6 <- coxph( st1 ~ DGN + strata(ag) + DIAB + MINF, data=stroke)
round( ci.lin(m6, Exp=T)[, 5:7], 4 )
cox.zph(m6)

## Alternatively: try just ordinary logistic regression and compare! 

m7 <- glm(event1 ~ ag + DGN+DIAB+MINF,data=stroke,family=binomial)
round(ci.lin(m7, Exp=T)[, 5:7], 4)

##################################################################### 
